@@ -4,19 +4,21 @@ use dim4::Dim4;
 use defines::{AfError, DType, Backend};
 use error::HANDLE_ERROR;
 use util::HasAfEnum;
-use self::libc::{uint8_t, c_void, c_int, c_uint, c_longlong};
+use self::libc::{uint8_t, c_void, c_int, c_uint, c_longlong, c_char};
+use std::ffi::CString;
 
 type MutAfArray = *mut self::libc::c_longlong;
 type MutDouble  = *mut self::libc::c_double;
 type MutUint    = *mut self::libc::c_uint;
 type AfArray    = self::libc::c_longlong;
 type DimT       = self::libc::c_longlong;
+type MutVoidPtr = *mut self::libc::c_ulonglong;
+type VoidPtr    = self::libc::c_ulonglong;
 
 // Some unused functions from array.h in C-API of ArrayFire
 // af_create_handle
 // af_copy_array
 // af_write_array
-// af_get_data_ptr
 // af_get_data_ref_count
 
 #[allow(dead_code)]
@@ -63,6 +65,12 @@ extern {
 
     fn af_eval(arr: AfArray) -> c_int;
 
+    fn af_eval_multiple(num: c_int, arrays: *const AfArray) -> c_int;
+
+    fn af_set_manual_eval_flag(flag: c_int) -> c_int;
+
+    fn af_get_manual_eval_flag(flag: *mut c_int) -> c_int;
+
     fn af_retain_array(out: MutAfArray, arr: AfArray) -> c_int;
 
     fn af_copy_array(out: MutAfArray, arr: AfArray) -> c_int;
@@ -71,6 +79,8 @@ extern {
 
     fn af_print_array(arr: AfArray) -> c_int;
 
+    fn af_print_array_gen(exp: *const c_char, arr: AfArray, precision: c_int) -> c_int;
+
     fn af_cast(out: MutAfArray, arr: AfArray, aftype: uint8_t) -> c_int;
 
     fn af_get_backend_id(backend: *mut c_int, input: AfArray) -> c_int;
@@ -79,7 +89,7 @@ extern {
 
     fn af_create_strided_array(arr: MutAfArray, data: *const c_void, offset: DimT,
                                ndims: c_uint, dims: *const DimT, strides: *const DimT,
-                               aftype: uint8_t) -> c_int;
+                               aftype: uint8_t, stype: uint8_t) -> c_int;
 
     fn af_get_strides(s0: *mut DimT, s1: *mut DimT, s2: *mut DimT, s3: *mut DimT,
                       arr: AfArray) -> c_int;
@@ -89,6 +99,14 @@ extern {
     fn af_is_linear(result: *mut c_int, arr: AfArray) -> c_int;
 
     fn af_is_owner(result: *mut c_int, arr: AfArray) -> c_int;
+
+    fn af_is_sparse(result: *mut c_int, arr: AfArray) -> c_int;
+
+    fn af_lock_array(arr: AfArray) -> c_int;
+
+    fn af_unlock_array(arr: AfArray) -> c_int;
+
+    fn af_get_device_ptr(ptr: MutVoidPtr, arr: AfArray) -> c_int;
 }
 
 /// A multidimensional data container
@@ -99,8 +117,8 @@ pub struct Array {
 }
 
 macro_rules! is_func {
-    ($fn_name: ident, $ffi_fn: ident) => (
-        /// Checks if the Array is of specific format/data type.
+    ($doc_str: expr, $fn_name: ident, $ffi_fn: ident) => (
+        #[doc=$doc_str]
         pub fn $fn_name(&self) -> bool {
             unsafe {
                 let mut ret_val: i32 = 0;
@@ -117,7 +135,7 @@ impl Array {
     ///
     /// # Examples
     ///
-    /// ```
+    /// ```rust
     /// use arrayfire::{Array, Dim4, print};
     /// let values: [f32; 3] = [1.0, 2.0, 3.0];
     /// let indices = Array::new(&values, Dim4::new(&[3, 1, 1, 1]));
@@ -153,7 +171,7 @@ impl Array {
                                                   dims.ndims() as c_uint,
                                                   dims.get().as_ptr() as * const c_longlong,
                                                   strides.get().as_ptr() as * const c_longlong,
-                                                  aftype as uint8_t);
+                                                  aftype as uint8_t, 1);
             HANDLE_ERROR(AfError::from(err_val));
             Array::from(temp)
         }
@@ -296,20 +314,20 @@ impl Array {
         }
     }
 
-    is_func!(is_empty, af_is_empty);
-    is_func!(is_scalar, af_is_scalar);
-    is_func!(is_row, af_is_row);
-    is_func!(is_column, af_is_column);
-    is_func!(is_vector, af_is_vector);
-    is_func!(is_complex, af_is_complex);
-    is_func!(is_double, af_is_double);
-    is_func!(is_single, af_is_single);
-    is_func!(is_real, af_is_real);
-    is_func!(is_floating, af_is_floating);
-    is_func!(is_integer, af_is_integer);
-    is_func!(is_bool, af_is_bool);
-    is_func!(is_linear, af_is_linear);
-    is_func!(is_owner, af_is_owner);
+    is_func!("Check if Array is empty", is_empty, af_is_empty);
+    is_func!("Check if Array is scalar", is_scalar, af_is_scalar);
+    is_func!("Check if Array is a row", is_row, af_is_row);
+    is_func!("Check if Array is a column", is_column, af_is_column);
+    is_func!("Check if Array is a vector", is_vector, af_is_vector);
+    is_func!("Check if Array is of complex type", is_complex, af_is_complex);
+    is_func!("Check if Array's numerical type is of double precision", is_double, af_is_double);
+    is_func!("Check if Array's numerical type is of single precision", is_single, af_is_single);
+    is_func!("Check if Array is of real type", is_real, af_is_real);
+    is_func!("Check if Array is of single precision", is_floating, af_is_floating);
+    is_func!("Check if Array is of integral type", is_integer, af_is_integer);
+    is_func!("Check if Array is of boolean type", is_bool, af_is_bool);
+    is_func!("Check if Array's memory layout is continuous and one dimensional", is_linear, af_is_linear);
+    is_func!("Check if Array's memory is owned by it and not a view of another Array", is_owner, af_is_owner);
 
     /// Cast the Array data type to `target_type`
     pub fn cast<T: HasAfEnum>(&self) -> Array {
@@ -321,6 +339,49 @@ impl Array {
             Array::from(temp)
         }
     }
+
+    /// Find if the current array is sparse
+    pub fn is_sparse(&self) -> bool {
+        unsafe {
+            let mut temp: i32 = 0;
+            let err_val = af_is_sparse(&mut temp as *mut c_int, self.handle as AfArray);
+            HANDLE_ERROR(AfError::from(err_val));
+            temp > 0
+        }
+    }
+
+    /// Lock the device buffer in the memory manager
+    ///
+    /// Locked buffers are not freed by memory manager until unlock is called.
+    pub fn lock(&self) {
+        unsafe {
+            let err_val = af_lock_array(self.handle as AfArray);
+            HANDLE_ERROR(AfError::from(err_val));
+        }
+    }
+
+    /// Unlock the device buffer in the memory manager
+    ///
+    /// This function will give back the control over the device pointer to the
+    /// memory manager.
+    pub fn unlock(&self) {
+        unsafe {
+            let err_val = af_unlock_array(self.handle as AfArray);
+            HANDLE_ERROR(AfError::from(err_val));
+        }
+    }
+
+    /// Get the device pointer and lock the buffer in memory manager
+    ///
+    /// The device pointer is not freed by memory manager until unlock is called.
+    pub fn device_ptr(&self) -> u64 {
+        unsafe {
+            let mut temp: u64 = 0;
+            let err_val = af_get_device_ptr(&mut temp as MutVoidPtr, self.handle as AfArray);
+            HANDLE_ERROR(AfError::from(err_val));
+            temp
+        }
+    }
 }
 
 /// Used for creating Array object from native resource id
@@ -359,19 +420,23 @@ impl Drop for Array {
 
 /// Print data in the Array
 ///
+/// # Parameters
+///
+/// - `input` is the Array to be printed
+///
 /// # Examples
 ///
-/// ```
+/// ```rust
 /// use arrayfire::{Dim4, print, randu};
 /// println!("Create a 5-by-3 matrix of random floats on the GPU");
-/// let dims = Dim4::new(&[3, 1, 1, 1]);
+/// let dims = Dim4::new(&[5, 3, 1, 1]);
 /// let a = randu::<f32>(dims);
 /// print(&a);
 /// ```
 ///
 /// The sample output will look like below:
 ///
-/// ```bash
+/// ```text
 /// [5 3 1 1]
 ///     0.7402     0.4464     0.7762
 ///     0.9210     0.6673     0.2948
@@ -380,8 +445,102 @@ impl Drop for Array {
 ///     0.9251     0.5132     0.6814
 /// ```
 pub fn print(input: &Array) {
+    let emptystring = CString::new("").unwrap();
     unsafe {
-        let err_val = af_print_array(input.get() as AfArray);
+        let err_val = af_print_array_gen(emptystring.to_bytes_with_nul().as_ptr() as *const c_char,
+                                         input.get() as AfArray, 4);
         HANDLE_ERROR(AfError::from(err_val));
     }
 }
+
+/// Generalized Array print function
+///
+/// Use this function to print Array data with arbitrary preicsion
+///
+/// # Parameters
+///
+/// - `msg` is message to be printed before printing the Array data
+/// - `input` is the Array to be printed
+/// - `precision` is data precision with which Array has to be printed
+///
+/// # Examples
+///
+/// ```rust
+/// use arrayfire::{Dim4, print_gen, randu};
+/// println!("Create a 5-by-3 matrix of random floats on the GPU");
+/// let dims = Dim4::new(&[5, 3, 1, 1]);
+/// let a = randu::<f32>(dims);
+/// print_gen(String::from("Random Array"), &a, Some(6));
+/// ```
+///
+/// The sample output will look like below:
+///
+/// ```text
+/// Random Array
+///
+/// [5 3 1 1]
+///     0.740276     0.446440     0.776202
+///     0.921094     0.667321     0.294810
+///     0.039014     0.109939     0.714090
+///     0.969058     0.470269     0.358590
+///     0.925181     0.513225     0.681451
+/// ```
+pub fn print_gen(msg: String, input: &Array, precision: Option<i32>) {
+    let emptystring = CString::new(msg.as_bytes()).unwrap();
+    unsafe {
+        let err_val = af_print_array_gen(emptystring.to_bytes_with_nul().as_ptr() as *const c_char,
+                                         input.get() as AfArray,
+                                         match precision {Some(p)=>p, None=>4} as c_int);
+        HANDLE_ERROR(AfError::from(err_val));
+    }
+}
+
+/// evaluate multiple arrays
+///
+/// Use this function to evaluate multiple arrays in single call
+///
+/// # Parameters
+///
+/// - `inputs` are the list of arrays to be evaluated
+pub fn eval_multiple(inputs: Vec<&Array>) {
+    unsafe {
+        let mut v = Vec::new();
+        for i in inputs {
+            v.push(i.get());
+        }
+
+        let err_val = af_eval_multiple(v.len() as c_int, v.as_ptr() as *const AfArray);
+        HANDLE_ERROR(AfError::from(err_val));
+    }
+}
+
+/// Set eval flag value
+///
+/// This function can be used to toggle on/off the manual evaluation of arrays.
+///
+/// # Parameters
+///
+/// - `flag` is a boolean value indicating manual evaluation setting
+pub fn set_manual_eval(flag: bool) {
+    unsafe {
+        let err_val = af_set_manual_eval_flag(flag as c_int);
+        HANDLE_ERROR(AfError::from(err_val));
+    }
+}
+
+/// Get eval flag value
+///
+/// This function can be used to find out if manual evaluation of arrays is
+/// turned on or off.
+///
+/// # Return Values
+///
+/// A boolean indicating manual evaluation setting.
+pub fn is_eval_manual() -> bool {
+    unsafe {
+        let mut ret_val: i32 = 0;
+        let err_val = af_get_manual_eval_flag(&mut ret_val as *mut c_int);
+        HANDLE_ERROR(AfError::from(err_val));
+        ret_val > 0
+    }
+}
